@@ -13,6 +13,7 @@ import lxml.etree
 import numpy as np
 from astropy import units as u
 from astropy.coordinates import SkyCoord
+from astropy.io.fits import getheader
 from celery import group
 from hpmoc import PartialUniqSkymap
 from hpmoc.utils import reraster, uniq_intersection
@@ -22,6 +23,7 @@ from ligo.skymap.moc import bayestar_adaptive_grid
 from ligo.skymap.plot.bayes_factor import plot_bayes_factor
 
 from .. import _version, app
+from ..util import closing_figures
 from ..util.cmdline import handling_system_exit
 from ..util.tempfile import NamedTemporaryFile
 from . import gracedb, skymaps
@@ -36,17 +38,14 @@ FERMI_OFFICIAL_SKYMAP_FILENAME = 'glg_healpix_all_bn_v00'
 """Filename of sky map from official Fermi GBM analysis"""
 
 NOTICE_TYPE_DICT = {
-        '53': 'INTEGRAL_WAKEUP',
-        '54': 'INTEGRAL_REFINED',
-        '55': 'INTEGRAL_OFFLINE',
         '60': 'SWIFT_BAT_GRB_ALERT',
         '61': 'SWIFT_BAT_GRB_POSITION',
-        '105': 'AGILE_MCAL_ALERT',
         '110': 'FERMI_GBM_ALERT',
         '111': 'FERMI_GBM_FLT_POS',
         '112': 'FERMI_GBM_GND_POS',
         '115': 'FERMI_GBM_FINAL_POS',
-        '131': 'FERMI_GBM_SUBTHRESHOLD'
+        '131': 'FERMI_GBM_SUBTHRESHOLD',
+        '202': 'SVOM_ECLAIRS_WAKEUP'
 }
 
 
@@ -159,6 +158,12 @@ def get_skymap_filename(graceid, is_gw):
     raise ValueError('No skymap available for {0} yet.'.format(graceid))
 
 
+def is_skymap_moc(skymap_bytes):
+    with NamedTemporaryFile(content=skymap_bytes) as skymap_file:
+        # If ordering is EXPLICIT, should be multiordered (MOC)
+        return getheader(skymap_file.name, ext=1)['INDXSCHM'] == 'EXPLICIT'
+
+
 @app.task(shared=False)
 def _download_skymaps(gw_filename, ext_filename, gw_id, ext_id):
     """Download both superevent and external sky map to be combined.
@@ -277,6 +282,9 @@ def combine_skymaps_moc_flat(gw_sky, ext_sky, ext_header):
     ext_ind = ah.lonlat_to_healpix(
         ra_gw, dec_gw, ext_nside,
         order='nested' if ext_header['nest'] else 'ring')
+    #  Fix any negative values and normalize
+    ext_sky[ext_sky < 0.] = 0.
+    ext_sky /= np.sum(ext_sky)
     #  Reweight GW prob density by external sky map probabilities
     gw_sky['PROBDENSITY'] *= ext_sky[ext_ind]
     gw_sky['PROBDENSITY'] /= \
@@ -341,7 +349,7 @@ def combine_skymaps(skymapsbytes, ext_moc=True):
         else:
             #  Load external sky map
             ext_skymap, ext_header = fits.read_sky_map(ext_skymap_file.name,
-                                                       moc=False)
+                                                       moc=False, nest=True)
             #  Create and write combined sky map
             combined_skymap = combine_skymaps_moc_flat(gw_skymap, ext_skymap,
                                                        ext_header)
@@ -378,8 +386,8 @@ def external_trigger_heasarc(external_id):
         external_id))
 
 
-@app.task(autoretry_for=(gracedb.RetryableHTTPError,), retry_backoff=10,
-          max_retries=14)
+@app.task(autoretry_for=(gracedb.RetryableHTTPError,), retry_backoff=30,
+          max_retries=8)
 def get_external_skymap(link, search):
     """Download the Fermi sky map FITS file and return the contents as a byte
     array. If GRB, will construct a HEASARC url, while if SubGRB, will use the
@@ -440,13 +448,15 @@ def read_upload_skymap_from_base64(event, skymap_str):
     """
 
     graceid = event['graceid']
-    ra = event['extra_attributes']['GRB']['ra']
-    dec = event['extra_attributes']['GRB']['dec']
-    skymap_filename = event['pipeline'].lower() + '_skymap.fits.gz'
 
     # Decode base64 encoded string to bytes string
     skymap_data = b64decode(skymap_str)
 
+    # Determine filename based on whether is multiordered or flattened
+    skymap_filename = event['pipeline'].lower() + '_skymap.'
+    skymap_filename += ('multiorder.fits' if is_skymap_moc(skymap_data)
+                        else 'fits.gz')
+
     message = (
         'Mollweide projection of <a href="/api/events/{graceid}/files/'
         '{filename}">{filename}</a>').format(
@@ -462,7 +472,7 @@ def read_upload_skymap_from_base64(event, skymap_str):
                     event['pipeline']),
                 ['sky_loc']),
 
-            skymaps.plot_allsky.si(skymap_data, ra=ra, dec=dec)
+            skymaps.plot_allsky.si(skymap_data)
             |
             gracedb.upload.s(event['pipeline'].lower() + '_skymap.png',
                              graceid,
@@ -496,17 +506,16 @@ def get_upload_external_skymap(event, skymap_link=None):
     search = event['search']
 
     if search == 'GRB':
-        external_skymap_canvas = (
-            external_trigger_heasarc.si(graceid)
-            |
-            get_external_skymap.s(search)
-        )
+        skymap_data = \
+            get_external_skymap(external_trigger_heasarc(graceid), search)
     elif search in {'SubGRB', 'SubGRBTargeted', 'FromURL'}:
-        external_skymap_canvas = get_external_skymap.si(skymap_link, search)
+        skymap_data = get_external_skymap(skymap_link, search)
 
-    skymap_filename = \
+    skymap_filename_base = \
         ('external_from_url' if search == 'FromURL'
          else FERMI_OFFICIAL_SKYMAP_FILENAME)
+    skymap_filename = skymap_filename_base + \
+        ('.multiorder.fits' if is_skymap_moc(skymap_data) else '.fits.gz')
 
     fits_message = \
         ('Downloaded from {}.'.format(skymap_link) if search == 'FromURL'
@@ -514,21 +523,20 @@ def get_upload_external_skymap(event, skymap_link=None):
     png_message = (
         'Mollweide projection of <a href="/api/events/{graceid}/files/'
         '{filename}">{filename}</a>').format(
-            graceid=graceid, filename=skymap_filename + '.fits.gz')
+            graceid=graceid, filename=skymap_filename)
 
     (
-        external_skymap_canvas
-        |
         group(
-            gracedb.upload.s(
-                skymap_filename + '.fits.gz',
+            gracedb.upload.si(
+                skymap_data,
+                skymap_filename,
                 graceid,
                 fits_message,
                 ['sky_loc']),
 
-            skymaps.plot_allsky.s()
+            skymaps.plot_allsky.si(skymap_data)
             |
-            gracedb.upload.s(skymap_filename + '.png',
+            gracedb.upload.s(skymap_filename_base + '.png',
                              graceid,
                              png_message,
                              ['sky_loc'])
@@ -662,7 +670,7 @@ def create_external_skymap(ra, dec, error, pipeline, notice_type=111):
                                         core_width, tail_width, core_weight,
                                         rounds=8)
     else:
-        # Use generic cone method for Swift, INTEGRAL, AGILE, etc.
+        # Use generic cone method for Swift, etc.
         skymap = bayestar_adaptive_grid(from_cone, ra, dec, error, rounds=8)
 
     return skymap
@@ -764,7 +772,7 @@ def create_upload_external_skymap(event, notice_type, notice_date):
                 extra_sentence),
             ['sky_loc'])
         |
-        skymaps.plot_allsky.si(skymap_data, ra=ra, dec=dec)
+        skymaps.plot_allsky.si(skymap_data)
         |
         gracedb.upload.s(event['pipeline'].lower() + '_skymap.png',
                          graceid,
@@ -776,6 +784,7 @@ def create_upload_external_skymap(event, notice_type, notice_date):
 
 
 @app.task(shared=False)
+@closing_figures()
 def plot_overlap_integral(coinc_far_dict, superevent, ext_event,
                           var_label=r"\mathcal{I}_{\Omega}"):
     """Plot and upload visualization of the sky map overlap integral computed
