@@ -24,14 +24,15 @@ def catch_retryable_http_errors(f):
     """Decorator to capture server-side errors that we should retry.
 
     We retry HTTP status 502 (Bad Gateway), 503 (Service Unavailable), and
-    504 (Gateway Timeout).
+    504 (Gateway Timeout). We also retry client side error codes 408 (Timeout),
+    409 (Conflicting URL), 429 (Too many requests).
     """
     @functools.wraps(f)
     def wrapper(*args, **kwargs):
         try:
             return f(*args, **kwargs)
         except HTTPError as e:
-            if e.response.status_code in {408, 429, 502, 503, 504}:
+            if e.response.status_code in {408, 409, 429, 500, 502, 503, 504}:
                 raise RetryableHTTPError(
                     *e.args, request=e.request, response=e.response)
             else:
@@ -96,6 +97,20 @@ def create_label(label, graceid):
             raise
 
 
+@task(ignore_result=True, shared=False)
+@catch_retryable_http_errors
+def create_label_with_log(log_message, label, tags, superevent_id):
+    """Create a label with a log for a superevent in GraceDB."""
+    try:
+        client.superevents[superevent_id].logs.create(
+            comment=log_message,
+            label=label,
+            tags=tags
+        )
+    except HTTPError:
+        raise
+
+
 @task(ignore_result=True, shared=False)
 @catch_retryable_http_errors
 def remove_label(label, graceid):
@@ -357,3 +372,39 @@ def add_event_to_superevent(superevent_id, graceid):
         if not (e.response.status_code == 400
                 and error_msg in e.response.content):
             raise
+
+
+@task(shared=False)
+@catch_retryable_http_errors
+def get_superevent_file_list(superevent_id):
+    """Get superevent file list from GraceDB."""
+    filelist = client.superevents[superevent_id].files.get()
+    return filelist
+
+
+@task(shared=False)
+@catch_retryable_http_errors
+def get_latest_file(superevent_id, filename):
+    """Get the lastest file provided a file name
+
+    Parameters
+    ----------
+    superevent_id : str
+        superevent uid
+
+    file_name : str
+        The filebase of a file name
+        e.g. 'bayestar.multiorder.fits' for 'bayestar.multiorder.fits,0'
+
+    Returns
+    ----------
+        The versioned filename for the inquired file
+    """
+    # Get file list for superevent
+    file_list = get_superevent_file_list(superevent_id)
+    # Loop over the keys and mark the key if key includes filename
+    keys = [key for key in file_list.keys() if filename in key]
+    if any(keys):
+        return max(keys)
+    else:
+        return None
