@@ -1,16 +1,16 @@
-from io import BytesIO
 import logging
+from io import BytesIO
 from pathlib import Path
 from unittest.mock import call, patch
 
-from astropy.time import Time
-from gwpy.timeseries import Bits
+import celery
 import matplotlib.pyplot as plt
 import numpy as np
 import pytest
+from astropy.time import Time
+from gwpy.timeseries import Bits
 
-from ..import app
-from ..import _version
+from .. import _version, app
 from ..tasks import detchar
 
 
@@ -26,6 +26,12 @@ def llhoft_glob_fail(monkeypatch):
     yield monkeypatch.setitem(app.conf, 'llhoft_glob', path)
 
 
+@pytest.fixture
+def llhoft_glob_idq_bad(monkeypatch):
+    path = str(Path(__file__).parent / 'data/llhoft/idqbad/{detector}/*.gwf')
+    yield monkeypatch.setitem(app.conf, 'llhoft_glob', path)
+
+
 @pytest.fixture
 def ifo_h1(monkeypatch):
     monkeypatch.setitem(app.conf, 'llhoft_channels', {
@@ -33,10 +39,33 @@ def ifo_h1(monkeypatch):
         'H1:GDS-CALIB_STATE_VECTOR': 'ligo_state_vector_bits'})
 
 
+@pytest.fixture
+def ifo_l1(monkeypatch):
+    monkeypatch.setitem(app.conf, 'llhoft_channels', {
+        'L1:DMT-DQ_VECTOR': 'dmt_dq_vector_bits',
+        'L1:GDS-CALIB_STATE_VECTOR': 'ligo_state_vector_bits'})
+
+
 @pytest.fixture
 def ifo_h1_idq(monkeypatch):
     monkeypatch.setitem(
-        app.conf, 'idq_channels', ['H1:IDQ-PGLITCH_OVL_32_2048'])
+        app.conf, 'idq_channels', ['H1:IDQ-FAP_OVL_32_2048'])
+    monkeypatch.setitem(
+        app.conf, 'idq_ok_channels', ['H1:IDQ-OK_OVL_32_2048'])
+
+
+@pytest.fixture
+def ifo_l1_idq(monkeypatch):
+    monkeypatch.setitem(
+        app.conf, 'idq_channels', ['L1:IDQ-FAP_OVL_32_2048'])
+    monkeypatch.setitem(
+        app.conf, 'idq_ok_channels', ['L1:IDQ-OK_OVL_32_2048'])
+
+
+@pytest.fixture
+def ifo_l1_idq_wrong_channel_name(monkeypatch):
+    monkeypatch.setitem(
+        app.conf, 'idq_channels', ['L1:IDQ-FAP_OVL_WRONG'])
 
 
 @pytest.fixture
@@ -67,28 +96,35 @@ def test_create_cache_old_data(mock_find, llhoft_glob_fail):
     mock_find.assert_called()
 
 
+@patch('gwcelery.tasks.detchar.find_urls', return_value=[])
+def test_create_cache_stale_devshm(mock_find, llhoft_glob_fail):
+    start, end = 1398800018, 1398800028
+    detchar.create_cache('L1', start, end)
+    mock_find.assert_called()
+
+
 expected_path = Path(__file__).parent / 'data/llhoft/omegascan/scanme.gwf'
 
 
 @patch('gwcelery.tasks.detchar.create_cache', return_value=[expected_path])
 def test_make_omegascan_worked(mock_create_cache, scan_strainname):
-    durs = [1, 1, 1]
+    durs = [(1, 1), (1, 1), (1, 1)]
     t0 = 1126259463
     png = detchar.make_omegascan('H1', t0, durs)
     pngarray = plt.imread(BytesIO(png))
-    # Test to see that the png is taller than 1400 pixels, indicating
+    # Test to see that the png is wider than 2000 pixels, indicating
     # presence of omegascan(s)
-    assert plt.imshow(pngarray).get_extent()[2] > 1400
+    assert plt.imshow(pngarray).get_extent()[1] > 850
 
 
 @patch('gwcelery.tasks.detchar.create_cache', return_value=[])
 def test_make_omegascan_failed(mock_create_cache, scan_strainname):
-    durs = [1, 1, 1]
+    durs = [(1, 1), (1, 1), (1, 1)]
     t0 = 1126259463
     png = detchar.make_omegascan('H1', t0, durs)
     pngarray = plt.imread(BytesIO(png))
-    # Test to see that the png is shorter than 1500 pixels
-    assert plt.imshow(pngarray).get_extent()[2] < 1500
+    # Test to see that the png is narrower than 2000 pixels
+    assert plt.imshow(pngarray).get_extent()[1] < 2000
 
 
 @patch('gwcelery.tasks.detchar.make_omegascan.run',
@@ -104,7 +140,17 @@ def test_omegascan(mock_upload, mock_fig):
     )
 
 
-def test_omegascan_skips_ew(caplog):
+def test_ifo_from_channel():
+    channels = [
+        'H1:GDS-CALIB_STRAIN', 'L1:DMT-DQ_VECTOR',
+        'V1:DQ_ANALYSIS_STATE_VECTOR', 'K1:TEST-CHAN']
+    ifos = [detchar.ifo_from_channel(chan) for chan in channels]
+    assert ifos == ['H1', 'L1', 'V1', 'K1']
+
+
+@patch('gwcelery.tasks.gracedb.upload.run')
+@patch('gwcelery.tasks.detchar.make_omegascan.run')
+def test_omegascan_skips_ew(mock_scan, mock_upload, caplog):
     """Test that omegascans are delayed for events in the future."""
     caplog.set_level(logging.INFO)
     detchar.omegascan(Time.now().gps + 99, 'S1234')
@@ -113,11 +159,22 @@ def test_omegascan_skips_ew(caplog):
 
 
 def test_check_idq(llhoft_glob_pass):
-    channel = 'H1:IDQ-PGLITCH_OVL_32_2048'
+    channel = 'H1:IDQ-FAP_OVL_32_2048'
     start, end = 1216577976, 1216577980
     cache = detchar.create_cache('H1', start, end)
     assert detchar.check_idq(cache, channel, start, end) == (
-        'H1:IDQ-PGLITCH_OVL_32_2048', 0)
+        'H1:IDQ-FAP_OVL_32_2048', 1)
+
+
+def test_check_idq_wrong_channel_handled(caplog, llhoft_glob_pass):
+    """Test that iDQ checks looking at the wrong channel are handled."""
+    caplog.set_level(logging.INFO)
+    channel = 'H1:IDQ-FAP_OVL_WRONG'
+    start, end = 1216577976, 1216577980
+    cache = detchar.create_cache('H1', start, end)
+    detchar.check_idq(cache, channel, start, end)
+    messages = [record.message for record in caplog.records]
+    assert 'Failed to read from low-latency iDQ frame files' in messages  # noqa: E501
 
 
 @patch('time.strftime', return_value='00:00:00 UTC Mon 01 Jan 2000')
@@ -227,11 +284,11 @@ def test_check_vectors(mock_create_label, mock_remove_label, mock_upload,
             ['data_quality']),
         call(
             None, None, 'S12345a',
-            ('iDQ glitch probabilities at both H1 and L1'
-             ' are good (below {} threshold). '
-             'Maximum p(glitch) is "H1:IDQ-PGLITCH_OVL_32_2048": 0.0. '
+            ('iDQ false alarm probabilities for active detectors'
+             ' are good (above {} threshold). '
+             'Minimum FAP is "H1:IDQ-FAP_OVL_32_2048": 1.0. '
              'Check looked within -1.5/+1.5 seconds of superevent. ').format(
-                 app.conf['idq_pglitch_thresh']),
+                 app.conf['idq_fap_thresh']),
             ['data_quality']),
         call(
             '"dqrjson"', 'gwcelerydetcharcheckvectors-S12345a.json', 'S12345a',
@@ -245,6 +302,89 @@ def test_check_vectors(mock_create_label, mock_remove_label, mock_upload,
         any_order=True)
 
 
+@patch('gwcelery.tasks.detchar.dqr_json', return_value='dqrjson')
+@patch('gwcelery.tasks.gracedb.upload.run')
+@patch('gwcelery.tasks.gracedb.remove_label')
+@patch('gwcelery.tasks.gracedb.create_label')
+def test_check_vectors_fails(
+        mock_create_label, mock_remove_label, mock_upload,
+        mock_json, llhoft_glob_fail, ifo_l1, ifo_l1_idq):
+    event = {'search': 'AllSky', 'instruments': 'L1', 'pipeline': 'oLIB'}
+    superevent_id = 'S12345a'
+    start, end = 1216577978, 1216577978.1
+    detchar.check_vectors(event, superevent_id, start, end)
+    calls = [
+        call(
+            None, None, 'S12345a',
+            ('Detector state for active instruments is bad.\n{}'
+             'Check looked within -1.5/+1.5 seconds of superevent. ').format(
+                 detchar.generate_table(
+                     'Data quality bits', [],
+                     ['L1:NO_OMC_DCPD_ADC_OVERFLOW',
+                      'L1:NO_DMT-ETMY_ESD_DAC_OVERFLOW',
+                      'L1:HOFT_OK', 'L1:OBSERVATION_INTENT'], [])),
+            ['data_quality']),
+        call(
+            None, None, 'S12345a',
+            ('Injection found.\n{}\n'
+             'Check looked within -1.5/+1.5 seconds of superevent. ').format(
+                 detchar.generate_table(
+                     'Injection bits', [],
+                     ['L1:NO_STOCH_HW_INJ', 'L1:NO_CBC_HW_INJ',
+                      'L1:NO_BURST_HW_INJ', 'L1:NO_DETCHAR_HW_INJ'], [])),
+            ['data_quality']),
+        call(
+            None, None, 'S12345a',
+            ('iDQ false alarm probability is low '
+             '(below {} threshold), '
+             'i.e., there could be a data quality issue: '
+             'minimum FAP is "L1:IDQ-FAP_OVL_32_2048": 0.0. '
+             'Check looked within -1.5/+1.5 seconds of superevent. ').format(
+                 app.conf['idq_fap_thresh']),
+            ['data_quality']),
+        call(
+            '"dqrjson"', 'gwcelerydetcharcheckvectors-S12345a.json', 'S12345a',
+            'DQR-compatible json generated from check_vectors results'),
+    ]
+    mock_upload.assert_has_calls(calls, any_order=True)
+    mock_create_label.assert_called_with('DQV', 'S12345a')
+    mock_remove_label.assert_called_with('DQOK', 'S12345a')
+
+
+@patch('gwcelery.tasks.gracedb.upload.run')
+def test_check_vectors_idq_not_ok(
+        mock_upload, llhoft_glob_idq_bad, ifo_h1, ifo_h1_idq):
+    event = {'search': 'AllSky', 'instruments': 'H1', 'pipeline': 'oLIB'}
+    superevent_id = 'S12345a'
+    start, end = 1216577978, 1216577978.1
+    detchar.check_vectors(event, superevent_id, start, end)
+    mock_upload.assert_any_call(
+        None, None, 'S12345a',
+        ('Not checking iDQ for H1 '
+            'because it has times where IDQ_OK = 0. '
+            'Check looked within -1.5/+1.5 seconds of superevent. '),
+        ['data_quality']
+    )
+
+
+@patch('gwcelery.tasks.gracedb.upload.run')
+@patch('celery.app.task.Task.request')
+@patch('gwcelery.tasks.detchar.check_vector', side_effect=ValueError)
+def test_check_vectors_retries_on_valueerror(
+        mock_check_vector, mock_request,
+        mock_upload, llhoft_glob_pass, ifo_h1, ifo_h1_idq):
+    # Mocking this retry https://docs.celeryq.dev/en/stable/_modules/celery/app/task.html#Task.retry  # noqa E501
+    event = {'search': 'AllSky', 'instruments': 'L1', 'pipeline': 'oLIB'}
+    superevent_id = 'S12345a'
+    start, end = 1216577978, 1216577978.1
+    mock_request.called_directly = False
+    mock_request.retries = 3
+    with pytest.raises(celery.exceptions.Retry) as retry_exc:
+        detchar.check_vectors.delay(event, superevent_id, start, end)
+    # after three retries, should still retry once in 5 seconds
+    assert retry_exc.value.when == 5
+
+
 @patch('gwcelery.tasks.detchar.dqr_json', return_value='dqrjson')
 @patch('gwcelery.tasks.gracedb.upload.run')
 @patch('gwcelery.tasks.gracedb.remove_label')
@@ -287,8 +427,6 @@ def test_detchar_generate_table():
               '<td bgcolor="yellow">unknown</td></tr>\n'
               '    <tr><td>unknown_bit_2</td>'
               '<td bgcolor="yellow">unknown</td></tr>\n'
-              '    <tr bgcolor="red"><th>Overall state for all '
-              'detectors</th><th>bad</th></tr>\n'
               '</table>')
     assert detchar.generate_table(
         'title', ['good_bit_1', 'good_bit_2'],
