@@ -48,6 +48,15 @@ def test_create_label(mock_gracedb):
         'label')
 
 
+@patch('gwcelery.tasks.gracedb.client')
+def test_create_label_with_log(mock_gracedb):
+    gracedb.create_label_with_log('log_message', 'label', ['tag'], 'graceid')
+    mock_gracedb.superevents['graceid'].logs.create.assert_called_once_with(
+        comment='log_message',
+        label='label',
+        tags=['tag'])
+
+
 @patch('gwcelery.tasks.gracedb.client')
 def test_remove_label(mock_gracedb):
     gracedb.remove_label('label', 'graceid')
