@@ -3,6 +3,7 @@ import glob
 import json
 import os
 import subprocess
+import sys
 from contextlib import nullcontext as does_not_raise
 from itertools import product
 from unittest.mock import Mock
@@ -135,11 +136,12 @@ def test_setup_dag_for_lalinference(monkeypatch, tmp_path):
     product(
         ['gracedb-playground.ligo.org', 'gracedb.ligo.org'],
         ['production', 'fast_test'],
-        [30, 10, 3, 2, 1, 0.1]
+        [30, 20, 15, 10, 3, 2, 1]
     )
 )
 def test_setup_dag_for_bilby(monkeypatch, tmp_path, host, mode, mc):
     psd = b'psd'
+    bayestar = b'bayestar'
     rundir = str(tmp_path)
     event = {'gpstime': 1187008882, 'graceid': 'G1234',
              'extra_attributes': {'CoincInspiral': {'mchirp': mc}}}
@@ -149,9 +151,6 @@ def test_setup_dag_for_bilby(monkeypatch, tmp_path, host, mode, mc):
     monkeypatch.setitem(app.conf, 'gracedb_host', host)
 
     def _subprocess_run(cmd, **kwargs):
-        if host != 'gracedb.ligo.org':
-            assert 'o3replay' in cmd
-
         assert cmd[2] == os.path.join(
             app.conf['pe_results_path'], sid, 'bilby', mode)
 
@@ -167,16 +166,30 @@ def test_setup_dag_for_bilby(monkeypatch, tmp_path, host, mode, mc):
         with open(path_to_psd, 'rb') as f:
             assert f.read() == psd
 
+        path_to_bayestar = cmd[10]
+        assert os.path.exists(path_to_bayestar)
+        with open(path_to_bayestar, 'rb') as f:
+            assert f.read() == bayestar
+
         if mode == "fast_test":
             assert "FastTest" in cmd
 
-        path_to_settings = cmd[10]
+        path_to_settings = cmd[12]
         assert os.path.exists(path_to_settings)
+
+        assert cmd[13] == "--llhoft-glob"
+        assert cmd[14] == app.conf['llhoft_glob']
+        channels = {
+            key: value.strip(f"{key}:") for key, value
+            in app.conf['strain_channel_names'].items()
+        }
         ans = {
             'summarypages_arguments': {'gracedb': event['graceid'],
                                        'no_ligo_skymap': True},
             'queue': 'Online_PE',
             'accounting_user': 'soichiro.morisaki',
+            'channel_dict': channels,
+            'tukey_roll_off': 1.0
         }
         if host != 'gracedb.ligo.org':
             ans['queue'] = 'Online_PE_MDC'
@@ -186,7 +199,7 @@ def test_setup_dag_for_bilby(monkeypatch, tmp_path, host, mode, mc):
             ans.update(
                 {'sampler_kwargs': {'naccept': 60, 'nlive': 500,
                                     'npool': 24, 'sample': 'acceptance-walk'},
-                 'n_parallel': 2,
+                 'n_parallel': 3,
                  'request_cpus': 24,
                  'spline_calibration_nodes': 10,
                  'request_memory_generation': 8.0}
@@ -200,8 +213,16 @@ def test_setup_dag_for_bilby(monkeypatch, tmp_path, host, mode, mc):
                 assert 'low_q_phenompv2_roq' in cmd
             else:
                 assert 'phenomxphm_roq' in cmd
-                ans['request_memory_generation'] = 36.0
-                ans['request_memory'] = 16.0
+                if mc > 16:
+                    ans['request_memory_generation'] = 36.0
+                else:
+                    ans['request_memory_generation'] = 50.0
+                if mc > 25:
+                    ans['request_memory'] = 16.0
+                elif mc > 16:
+                    ans['request_memory'] = 24.0
+                else:
+                    ans['request_memory'] = 36.0
         elif mode == 'fast_test' and mc < 3.9:
             ans['request_memory_generation'] = 8.0
         with open(path_to_settings, 'r') as f:
@@ -221,18 +242,14 @@ def test_setup_dag_for_bilby(monkeypatch, tmp_path, host, mode, mc):
     monkeypatch.setattr('subprocess.run', _subprocess_run)
     monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
 
-    if mc < 0.6:
-        with pytest.raises(ValueError):
-            inference._setup_dag_for_bilby(psd, rundir, event, sid, mode)
-    else:
-        path_to_dag = inference._setup_dag_for_bilby(
-            psd, rundir, event, sid, mode
-        )
+    path_to_dag = inference._setup_dag_for_bilby(
+        (psd, bayestar), rundir, event, sid, mode
+    )
 
-        assert os.path.exists(path_to_dag)
-        with open(path_to_dag, 'r') as f:
-            assert f.read() == dag
-        upload.assert_called_once()
+    assert os.path.exists(path_to_dag)
+    with open(path_to_dag, 'r') as f:
+        assert f.read() == dag
+    upload.assert_called_once()
 
 
 def test_setup_dag_for_bilby_unknown_mode(tmp_path):
@@ -249,7 +266,7 @@ def test_setup_dag_for_bilby_unknown_mode(tmp_path):
     'returncode,expectation',
     [
         (0, does_not_raise()),
-        (1, pytest.raises(subprocess.CalledProcessError)),
+        (1, pytest.raises(RuntimeError)),
         (inference._RAPIDPE_NO_GSTLAL_TRIGGER_EXIT_CODE,
          pytest.raises(Ignore)),
     ],
@@ -270,21 +287,19 @@ def test_setup_dag_for_rapidpe(
     event = {'superevent_id': 'S1234',
              'extra_attributes': {'CoincInspiral': {'snr': 10}}}
 
-    def _subprocess_run(args, *, capture_output=False, check=False):
+    def _run_rapidpe():
         # Simulate failed subprocess
         if returncode != 0:
-            raise subprocess.CalledProcessError(
-                returncode, args,
-                output=b"STDOUT", stderr=b"STDERR",
-            )
+            sys.exit(returncode)
 
-        path_to_ini = args[1]
+        path_to_ini = sys.argv[1]
         assert os.path.exists(path_to_ini)
         with open(os.path.join(rundir, dag_filename), 'w') as f:
             f.write(dag_content)
 
     upload = Mock()
-    monkeypatch.setattr('subprocess.run', _subprocess_run)
+    monkeypatch.setattr('gwcelery.tasks.inference.rapidpe_rift_pipe_main',
+                        _run_rapidpe)
     monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
     monkeypatch.setitem(app.conf, 'gracedb_host', host)
     with expectation:
@@ -304,10 +319,16 @@ def test_setup_dag_for_rapidpe(
 @pytest.mark.parametrize('pipeline', ['lalinference', 'bilby', 'rapidpe'])
 def test_setup_dag_for_failure(monkeypatch, tmp_path, pipeline):
     upload = Mock()
+    system_exit_from_runtime_error = RuntimeError(
+        'Command-line tool tried to exit with nonzero status'
+    )
+    system_exit_from_runtime_error.__cause__ = SystemExit(1)
     monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', upload)
     monkeypatch.setattr('subprocess.run',
                         Mock(side_effect=subprocess.CalledProcessError(
                             1, ['cmd'], output=b'out', stderr=b'err')))
+    monkeypatch.setattr('gwcelery.tasks.inference.rapidpe_rift_pipe_main',
+                        Mock(side_effect=system_exit_from_runtime_error))
     monkeypatch.setattr(
         'gwcelery.tasks.inference._find_appropriate_cal_env',
         Mock(return_value='./cal_file.txt'))
@@ -320,15 +341,18 @@ def test_setup_dag_for_failure(monkeypatch, tmp_path, pipeline):
                              [{'mass1': 1.4, 'mass2': 1.4, 'mchirp': 1.2}],
                              'CoincInspiral': {'mchirp': 1, 'snr': 10}}
     }
-    with pytest.raises(subprocess.CalledProcessError):
-        if pipeline == 'lalinference':
+    if pipeline == 'lalinference':
+        with pytest.raises(subprocess.CalledProcessError):
             inference._setup_dag_for_lalinference(
                 b'coinc', rundir, event, 'S1234')
-        elif pipeline == 'bilby':
+    elif pipeline == 'bilby':
+        with pytest.raises(subprocess.CalledProcessError):
             inference._setup_dag_for_bilby(
-                (b'psd'), rundir, event, 'S1234', 'production')
-        elif pipeline == 'rapidpe':
+                (b'psd', b'bayestar'), rundir, event, 'S1234', 'production')
+    elif pipeline == 'rapidpe':
+        with pytest.raises(RuntimeError):
             inference._setup_dag_for_rapidpe(rundir, 'S1234', event)
+
     if pipeline == 'bilby':
         assert upload.call_count == 1
     else:
@@ -342,6 +366,7 @@ def test_setup_dag_for_failure(monkeypatch, tmp_path, pipeline):
 def test_dag_prepare_task(monkeypatch, pipeline):
     sid = 'S1234'
     coinc = b'coinc'
+    bayestar = b'bayestar'
     event = {
             'gpstime': 1187008882,
             'graceid': 'G1234',
@@ -356,14 +381,17 @@ def test_dag_prepare_task(monkeypatch, pipeline):
     def mock_download(filename, gid):
         if filename == 'coinc.xml':
             return coinc
+        elif filename == 'bayestar.multiorder.fits':
+            return bayestar
 
     def _setup_dag_for_lalinference(c, r, e, s):
         assert (c == coinc and r == rundir and e == event and s == sid)
         return path_to_dag
 
-    def _setup_dag_for_bilby(c, r, e, s, m):
-        assert c == coinc and r == rundir and e == event and s == sid and \
-            m == kwargs['bilby_mode']
+    def _setup_dag_for_bilby(c_b, r, e, s, m):
+        c, b = c_b
+        assert c == coinc and b == bayestar and r == rundir and e == event \
+            and s == sid and m == kwargs['bilby_mode']
         return path_to_dag
 
     def _setup_dag_for_rapidpe(r, s, e):
@@ -610,15 +638,16 @@ def test_start_pe(monkeypatch, tmp_path, pipeline):
         for event_pipeline in event_pipeline_info:
             dag_prepare_task = Mock(return_value=mock_task.s())
 
-            condor_submit = Mock(side_effect=mock_condor_submit)
+            submit_rapidpe = Mock(side_effect=mock_condor_submit)
             dag_finished = Mock()
             monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', Mock())
-            monkeypatch.setattr('os.path.expanduser', Mock(
-                return_value=str(tmp_path)))
+            monkeypatch.setattr('platformdirs.user_cache_dir',
+                                Mock(return_value=str(tmp_path)))
             monkeypatch.setattr('gwcelery.tasks.inference.dag_prepare_task',
                                 dag_prepare_task)
             monkeypatch.setattr(
-                    'gwcelery.tasks.condor.submit_rapidpe.run', condor_submit)
+                    'gwcelery.tasks.inference.submit_rapidpe.run',
+                    submit_rapidpe)
             monkeypatch.setattr(
                     'gwcelery.tasks.inference.dag_finished.run',
                     dag_finished)
@@ -629,7 +658,7 @@ def test_start_pe(monkeypatch, tmp_path, pipeline):
                 'extra_attributes': {'CoincInspiral': {'snr': 10}}},
                 event_pipeline_info[event_pipeline]['sid'], pipeline)
             dag_prepare_task.assert_called_once()
-            condor_submit.assert_called_once()
+            submit_rapidpe.assert_called_once()
             dag_finished.assert_called_once()
 
     else:
@@ -640,7 +669,7 @@ def test_start_pe(monkeypatch, tmp_path, pipeline):
                              name="condor_submit")
         dag_finished = Mock(name="dag_finished")
         monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', Mock())
-        monkeypatch.setattr('distutils.dir_util.mkpath',
+        monkeypatch.setattr('platformdirs.user_cache_dir',
                             Mock(return_value=str(tmp_path)))
         monkeypatch.setattr('gwcelery.tasks.inference.dag_prepare_task',
                             dag_prepare_task)
@@ -649,7 +678,14 @@ def test_start_pe(monkeypatch, tmp_path, pipeline):
                 'gwcelery.tasks.inference.dag_finished.run',
                 dag_finished)
 
-        inference.start_pe({'graceid': 'G1234'}, 'S1234', pipeline)
+        inference.start_pe(
+            {
+                'graceid': 'G1234',
+                'extra_attributes': {'CoincInspiral': {'mchirp': 1.2}}
+            },
+            'S1234',
+            pipeline
+        )
         dag_prepare_task.assert_called_once()
         condor_submit.assert_called_once()
         dag_finished.assert_called_once()
@@ -659,7 +695,8 @@ def test_start_pe(monkeypatch, tmp_path, pipeline):
     'host', ["gracedb-playground.ligo.org", "gracedb.ligo.org"])
 def test_bbh_rate_limit(monkeypatch, tmp_path, host):
     monkeypatch.setitem(app.conf, 'gracedb_host', host)
-    monkeypatch.setattr('os.path.expanduser', Mock(return_value=str(tmp_path)))
+    monkeypatch.setattr(
+        'platformdirs.user_cache_dir', Mock(return_value=str(tmp_path)))
     event = {'gpstime': 1187008882, 'graceid': 'G1234',
              'extra_attributes': {'CoincInspiral': {'mchirp': 20}}}
     for i in range(6):
@@ -687,7 +724,7 @@ def test_bbh_rate_limit(monkeypatch, tmp_path, host):
     condor_submit = Mock(side_effect=mock_condor_submit)
     dag_finished = Mock()
     monkeypatch.setattr('gwcelery.tasks.gracedb.upload.run', Mock())
-    monkeypatch.setattr('distutils.dir_util.mkpath',
+    monkeypatch.setattr('platformdirs.user_cache_dir',
                         Mock(return_value=str(tmp_path)))
     monkeypatch.setattr('gwcelery.tasks.inference.dag_prepare_task',
                         dag_prepare_task)
@@ -733,6 +770,9 @@ def test_accounting_group_rapidpe(monkeypatch, tmp_path, conf):
     # Set app.conf based on gracedb_host
     monkeypatch.setattr(
             app.conf, 'rapidpe_settings', conf.rapidpe_settings)
+    # Mock running the pipeline generation code
+    monkeypatch.setattr('gwcelery.tasks.inference.rapidpe_rift_pipe_main',
+                        Mock())
 
     # Confirm the config file doesn't exist yet
     assert not os.path.isfile(config_path)
@@ -767,6 +807,9 @@ def test_environment_rapidpe(monkeypatch, tmp_path):
 
     # Set the GraceDB host appropriately
     monkeypatch.setitem(app.conf, 'gracedb_host', gracedb_host)
+    # Mock running the pipeline generation code
+    monkeypatch.setattr('gwcelery.tasks.inference.rapidpe_rift_pipe_main',
+                        Mock())
 
     # Generates the config file
     inference.dag_prepare_task(rundir, event, superevent_id, pe_pipeline).run()
