@@ -1,4 +1,4 @@
-from unittest.mock import call, patch
+from unittest.mock import Mock, call, patch
 
 import pytest
 
@@ -9,7 +9,7 @@ from .test_tasks_skymaps import toy_fits_filecontents  # noqa: F401
 
 @pytest.mark.live_worker
 @pytest.mark.parametrize(
-    'group,gracedb_id,pipelines,ext_search,se_search,tl,th',
+    'group,gracedb_id,pipelines,se_search,ext_search,tl,th',
     [['CBC', 'S1', ['Fermi', 'Swift'], [], ['GRB'], -1, 5],
      ['Burst', 'S2', ['Fermi', 'Swift'], [], ['GRB'], -60, 600],
      ['Burst', 'S3', ['SNEWS'], ['Supernova'], [], -10, 10],
@@ -17,9 +17,10 @@ from .test_tasks_skymaps import toy_fits_filecontents  # noqa: F401
      ['CBC', 'MS4', ['Fermi'], ['MDC'], [], -1, 5],
      ['CBC', 'E1', ['Fermi'], [], ['GRB'], -5, 1],
      ['CBC', 'M1', ['Fermi'], [], ['MDC'], -5, 1],
-     ['CBC', 'E1', ['Fermi'], ['SubGRB'], [], -11, 1],
-     ['Burst', 'E1', ['Swift'], ['SubGRBTargeted'], [], -20, 10],
-     [None, 'E1', ['Swift'], ['SubGRBTargeted'], [], -20, 10]])
+     ['CBC', 'E1', ['Fermi'], [], ['SubGRB'], -11, 1],
+     ['Burst', 'E1', ['Swift'], [], ['SubGRBTargeted'], -20, 10],
+     ['Burst', 'E1', ['Swift'], ['BBH'], [], -5, 1],
+     [None, 'E1', ['Swift'], [], ['SubGRBTargeted'], -20, 10]])
 @patch('gwcelery.tasks.gracedb.create_label')
 @patch('gwcelery.tasks.raven.raven_pipeline.s')
 @patch('gwcelery.tasks.raven.search.si', return_value=[{'superevent_id': 'S5',
@@ -29,7 +30,7 @@ def test_coincidence_search(mock_calculate_coincidence_far,
                             mock_search, mock_raven_pipeline,
                             mock_create_label,
                             group, gracedb_id, pipelines,
-                            ext_search, se_search, tl, th):
+                            se_search, ext_search, tl, th):
     """Test that correct time windows are used for each RAVEN search."""
     alert_object = {'superevent_id': gracedb_id}
     if 'E' in gracedb_id:
@@ -48,7 +49,7 @@ def test_coincidence_search(mock_calculate_coincidence_far,
     'group,search', [['CBC', 'SubGRBTargeted'], ['Test', 'GRB']])
 def test_raven_window_errors(group, search):
     with pytest.raises(ValueError):
-        raven._time_window('S1', group, ['INTEGRAL'], [search])
+        raven._time_window('S1', group, ['INTEGRAL'], [search], [])
 
 
 @pytest.mark.parametrize(
@@ -86,9 +87,11 @@ def test_calculate_coincidence_far(
     mock_calc_signif.assert_called_once_with(
         'S1234', 'E4321', tl, th,
         se_dict=se, ext_dict=ext,
-        incl_sky=False, grb_search='GRB',
+        incl_sky=False, grb_search=ext['search'],
+        em_rate=app.conf['raven_ext_rates'][ext['search']],
         gracedb=gracedb.client, far_grb=None,
         far_gw_thresh=None, far_grb_thresh=None)
+    assert isinstance(app.conf['raven_ext_rates'][ext['search']], float)
 
 
 @patch('ligo.raven.search.calc_signif_gracedb')
@@ -104,12 +107,17 @@ def test_calculate_coincidence_far_subgrb(mock_calc_signif):
     mock_calc_signif.assert_called_once_with(
         'S1234', 'E4321', tl, th,
         se_dict=se, ext_dict=ext,
-        incl_sky=False, grb_search='SubGRBTargeted',
+        incl_sky=False, grb_search=ext['search'],
         gracedb=gracedb.client, far_grb=ext['far'],
+        em_rate=None,
         far_gw_thresh=(
             app.conf['raven_targeted_far_thresholds']['GW']['Fermi']),
         far_grb_thresh=(
             app.conf['raven_targeted_far_thresholds']['GRB']['Fermi']))
+    assert isinstance(
+        app.conf['raven_targeted_far_thresholds']['GW']['Fermi'], float)
+    assert isinstance(
+        app.conf['raven_targeted_far_thresholds']['GRB']['Fermi'], float)
 
 
 @pytest.mark.parametrize('group', ['CBC', 'Burst'])  # noqa: F811
@@ -131,14 +139,16 @@ def test_calculate_spacetime_coincidence_far_fermi(
     raven.calculate_coincidence_far(se, ext, tl, th)
     mock_calc_signif.assert_called_once_with(
         'S1234', 'E4321', tl, th,
-        incl_sky=True, grb_search='GRB',
+        incl_sky=True, grb_search=ext['search'],
         se_dict=se, ext_dict=ext,
         se_fitsfile='fermi_skymap.fits.gz',
         ext_fitsfile='fermi_skymap.fits.gz',
         se_moc=True, ext_moc=False,
+        em_rate=app.conf['raven_ext_rates'][ext['search']],
         gracedb=gracedb.client, far_grb=None,
         far_gw_thresh=None, far_grb_thresh=None,
         use_preferred_event_skymap=False)
+    assert isinstance(app.conf['raven_ext_rates'][ext['search']], float)
 
 
 @pytest.mark.parametrize('group', ['CBC', 'Burst'])  # noqa: F811
@@ -160,14 +170,16 @@ def test_calculate_spacetime_coincidence_far_swift(
     raven.calculate_coincidence_far(se, ext, tl, th)
     mock_calc_signif.assert_called_once_with(
         'S1234', 'E4321', tl, th,
-        incl_sky=True, grb_search='GRB',
+        incl_sky=True, grb_search=ext['search'],
         se_dict=se, ext_dict=ext,
         se_fitsfile='swift_skymap.multiorder.fits',
         ext_fitsfile='swift_skymap.multiorder.fits',
+        em_rate=app.conf['raven_ext_rates'][ext['search']],
         se_moc=True, ext_moc=True,
         gracedb=gracedb.client, far_grb=None,
         far_gw_thresh=None, far_grb_thresh=None,
         use_preferred_event_skymap=False)
+    assert isinstance(app.conf['raven_ext_rates'][ext['search']], float)
 
 
 @pytest.mark.parametrize('group', ['CBC', 'Burst'])  # noqa: F811
@@ -190,14 +202,16 @@ def test_calculate_spacetime_coincidence_far_preferred(
     raven.calculate_coincidence_far(se, ext, tl, th)
     mock_calc_signif.assert_called_once_with(
         'S1234', 'E4321', tl, th,
-        incl_sky=True, grb_search='GRB',
+        incl_sky=True, grb_search=ext['search'],
         se_dict=se, ext_dict=ext,
         se_fitsfile='fermi_skymap.fits.gz',
         ext_fitsfile='fermi_skymap.fits.gz',
+        em_rate=app.conf['raven_ext_rates'][ext['search']],
         se_moc=True, ext_moc=False,
         gracedb=gracedb.client, far_grb=None,
         far_gw_thresh=None, far_grb_thresh=None,
         use_preferred_event_skymap=True)
+    assert isinstance(app.conf['raven_ext_rates'][ext['search']], float)
 
 
 @patch('ligo.raven.search.calc_signif_gracedb')
@@ -244,6 +258,9 @@ def mock_get_superevent(superevent_id):
     elif superevent_id == 'S5':
         old_time_far = 1e-4
         old_space_far = 1e-6
+    elif superevent_id == 'S6':
+        old_time_far = -1e-4
+        old_space_far = -1e-6
     else:
         old_time_far = 1e-5
         old_space_far = None
@@ -251,21 +268,23 @@ def mock_get_superevent(superevent_id):
             'space_coinc_far': old_space_far,
             'superevent_id': superevent_id,
             'labels': [],
-            'em_type': None}
+            'em_type': 'E101' if superevent_id != 'S101' else None}
 
 
 @pytest.mark.parametrize(
     'raven_search_results,graceid,tl,th,group',
-    [[[{'graceid': 'E1', 'pipeline': 'GRB'}], 'S1', -5, 1, 'CBC'],
+    [[[{'graceid': 'E1', 'search': 'GRB'}], 'S1', -5, 1, 'CBC'],
      [[{'superevent_id': 'S10', 'far': 1, 'preferred_event': 'G1'}],
         'E2', -1, 5, 'CBC'],
-     [[{'graceid': 'E3', 'pipeline': 'GRB'},
-       {'graceid': 'E4', 'pipeline': 'GRB'}], 'S2', -600, 60, 'Burst'],
+     [[{'graceid': 'E3', 'search': 'GRB'},
+       {'graceid': 'E4', 'search': 'GRB'}], 'S2', -600, 60, 'Burst'],
      [[{'superevent_id': 'S11', 'far': 1, 'preferred_event': 'G2'},
        {'superevent_id': 'S12', 'far': .001, 'preferred_event': 'G3'}],
         'E5', -1, 5, 'CBC'],
      [[], 'S13', -1, 5, 'CBC'],
-     [[{'graceid': 'E4', 'pipeline': 'GRB'}], 'S14', -1, 5, 'CBC'],
+     [[{'graceid': 'E4', 'search': 'GRB'}], 'S14', -1, 5, 'CBC'],
+     [[{'graceid': 'E5', 'search': 'SubGRBTargeted'}], 'S14', -1, 5, 'CBC'],
+     [[{'graceid': 'E7', 'search': 'SubGRBTargeted'}], 'S14', -1, 5, 'CBC'],
      [[{'superevent_id': 'S12', 'far': .001, 'preferred_event': 'G3'}],
         'T4', -10, 10, 'Burst'],
      [[{'superevent_id': 'MS13', 'far': 1, 'preferred_event': 'M3'}],
@@ -293,7 +312,13 @@ def test_raven_pipeline(mock_create_label,
                     'superevent': None if graceid != 'E6' else 'S14'}
 
     for result in raven_search_results:
-        result['labels'] = []
+        # Check if is an external event and is SubGRBTargeted, picking one
+        # to be low-significance
+        if result.get('graceid') and result['search'] == 'SubGRBTargeted' and \
+                result['graceid'] == 'E5':
+            result['labels'] = 'NOT_GRB'
+        else:
+            result['labels'] = []
     if 'S' not in graceid:
         alert_object['group'] = 'External'
         if 'T' in graceid:
@@ -346,7 +371,11 @@ def test_raven_pipeline(mock_create_label,
 
         mock_calculate_coincidence_far.assert_has_calls(coinc_calls,
                                                         any_order=True)
-        mock_create_label.assert_has_calls(label_calls, any_order=True)
+        if result.get('graceid') and result['search'] == 'SubGRBTargeted' and \
+                result['graceid'] == 'E5':
+            mock_create_label.assert_not_called()
+        else:
+            mock_create_label.assert_has_calls(label_calls, any_order=True)
     else:
         mock_trigger_raven_alert.assert_not_called()
         mock_calculate_coincidence_far.assert_not_called()
@@ -376,30 +405,55 @@ def test_preferred_superevent(raven_search_results, testnum):
 
 
 @pytest.mark.parametrize(
-    'new_time_far,new_space_far,superevent_id,pipeline,result',
-    [[1e-4, None, 'S1', 'Fermi', True],
-     [1e-4, 1e-3, 'S1', 'Swift', True],
-     [1e-4, None, 'S2', 'AGILE', False],
-     [1e-4, 1e-3, 'S3', 'Fermi', True],
-     [1e-4, 1e-3, 'S4', 'Fermi', False],
-     [1e-8, None, 'S4', 'Swift', False],
-     [1e-4, 1e-8, 'S5', 'AGILE', True],
-     [None, None, 'S1', 'SNEWS', True]])
-@patch('gwcelery.tasks.gracedb.update_superevent')
-@patch('gwcelery.tasks.gracedb.get_superevent', mock_get_superevent)
-def test_update_superevent(mock_update_superevent,
+    'new_time_far,new_space_far,superevent_id,search,result',
+    [[1e-4, None, 'S1', 'GRB', True],
+     [1e-4, 1e-3, 'S1', 'GRB', True],
+     [1e-4, None, 'S2', 'GRB', False],
+     [1e-4, 1e-3, 'S3', 'GRB', True],
+     [1e-4, 1e-3, 'S4', 'GRB', False],
+     [1e-8, None, 'S4', 'GRB', False],
+     [1e-4, 1e-8, 'S5', 'GRB', True],
+     # Check that ~zero temporal FAR is handled correctly
+     [1e-1000, None, 'S2', 'GRB', True],
+     # Check that ~zero spatial FAR is handled correctly
+     [1e-100, 1e-1000, 'S4', 'GRB', True],
+     # Check that positive values are preferred over negative
+     [1e-3, 1e-5, 'S6', 'GRB', True],
+     # Check that negative values are not preferred over positive
+     [-1e-7, -1e-10, 'S4', 'GRB', False],
+     [1e-10, 1e-15, 'S5', 'SubGRB', False],
+     [1e-10, 1e-15, 'S5', 'SubGRBTargeted', False],
+     [None, None, 'S1', 'Supernova', True]])
+def test_update_superevent(monkeypatch,
                            new_time_far, new_space_far,
                            superevent_id,
-                           pipeline, result):
+                           search, result):
     """This function tests that new coincidences update the superevent when
     they have a superior joint FAR or are a SNEWS event.
     """
+
+    def _mock_get_ext_event(graceid):
+        return {'graceid': graceid,
+                'pipeline': 'Fermi',
+                'search': search if graceid == 'E100' else 'GRB',
+                'labels': []}
+
+    mock_update_superevent = Mock()
+    monkeypatch.setattr('gwcelery.tasks.gracedb.update_superevent',
+                        mock_update_superevent)
+    monkeypatch.setattr('gwcelery.tasks.gracedb.get_superevent',
+                        mock_get_superevent)
+    monkeypatch.setattr('gwcelery.tasks.gracedb.get_event',
+                        _mock_get_ext_event)
+
     superevent = {'superevent_id': superevent_id,
                   'labels': []}
-    ext_event = {'graceid': 'E100',
-                 'pipeline': pipeline}
-    coinc_far_dict = {'temporal_coinc_far': new_time_far,
-                      'spatiotemporal_coinc_far': new_space_far}
+    ext_event = _mock_get_ext_event('E100')
+    if search == 'Supernova':
+        coinc_far_dict = {}
+    else:
+        coinc_far_dict = {'temporal_coinc_far': new_time_far,
+                          'spatiotemporal_coinc_far': new_space_far}
     raven.update_coinc_far(coinc_far_dict, superevent, ext_event)
     if result:
         mock_update_superevent.assert_called_with(
@@ -422,23 +476,33 @@ def mock_get_event(graceid):
             pipeline, search, labels = 'Fermi', 'GRB', []
         elif graceid == "E5":
             pipeline, search, labels = 'Fermi', 'GRB', ['RAVEN_ALERT']
+        elif graceid == "E6":
+            pipeline, search, labels = \
+                'Fermi', 'GRB', ['NOT_GRB', 'RAVEN_ALERT']
+        elif graceid == "E7":
+            pipeline, search, labels = 'Fermi', 'GRB', []
+        elif graceid is None:
+            return {}
         return {'superevent_id': graceid.replace('E', 'S'),
                 'graceid': graceid,
                 'pipeline': pipeline,
                 'search': search,
                 'labels': labels}
     elif 'S' in graceid:
-        return {'em_type': graceid.replace('S', 'E'),
-                'time_coinc_far': 1e-20,
-                'space_coinc_far': 1e-20,
+        return {'em_type': (graceid.replace('S', 'E') if graceid != 'S0'
+                            else None),
+                'time_coinc_far': 1e-20 if gracedb != 'S0' else None,
+                'space_coinc_far': 1e-20 if gracedb != 'S0' else None,
                 'superevent_id': graceid,
                 'labels': ['COMBINEDSKYMAP_READY'] if graceid == 'S4' else []}
 
 
 @pytest.mark.parametrize(
     'superevent_id,ext_event_id_old,ext_event_id_new,result',
-    # NOT_GRB should not supersede real GRB
-    [['S1', 'E1', 'E2', False],
+    # New event should always update if no previous event
+    [['S0', None, 'E1', True],
+     # NOT_GRB should not supersede real GRB
+     ['S1', 'E1', 'E2', False],
      # Real GRB should supersede NOT_GRB
      ['S2', 'E2', 'E1', True],
      # SNEWS event should not be overwritten by GRB
@@ -446,7 +510,13 @@ def mock_get_event(graceid):
      # RAVEN_ALERT should supersede subthreshold joint candidate
      ['S4', 'E4', 'E5', True],
      # Subthreshold joint candidate should not overwrite RAVEN_ALERT
-     ['S5', 'E5', 'E4', False]]
+     ['S5', 'E5', 'E4', False],
+     # New real event should supercede old NOT_GRB with an alert
+     ['S6', 'E6', 'E1', True],
+     # Same event graceid should overrwrite previous info
+     ['S1', 'E1', 'E1', True],
+     # Same event graceid should be ignored if no new spatial joint FAR
+     ['S7', 'E7', 'E7', False]]
 )
 @patch('gwcelery.tasks.gracedb.update_superevent')
 @patch('gwcelery.tasks.gracedb.get_event', mock_get_event)
@@ -463,10 +533,14 @@ def test_update_superevent_w_emtype(mock_update_superevent,
     """
     superevent = mock_get_event(superevent_id)
     superevent['em_type'] = ext_event_id_old
+    if ext_event_id_old:
+        ext_event_old = mock_get_event(ext_event_id_old)
+    else:
+        ext_event_old = {'labels': []}
     ext_event_new = mock_get_event(ext_event_id_new)
-    ext_event_old = mock_get_event(ext_event_id_old)
     time_coinc_far = 1e-10 if ext_event_old['labels'] else 1e-20
-    space_coinc_far = 1e-10 if ext_event_old['labels'] else 1e-30
+    space_coinc_far = (1e-10 if ext_event_old['labels'] else None if
+                       ext_event_id_old == 'E7' else 1e-30)
     coinc_far_dict = {'temporal_coinc_far': time_coinc_far,
                       'spatiotemporal_coinc_far': space_coinc_far}
     raven.update_coinc_far(coinc_far_dict, superevent, ext_event_new)
@@ -484,47 +558,52 @@ def _mock_get_event(graceid):
     if graceid == "S1234":
         return {"superevent_id": "S1234",
                 "preferred_event": "G000001",
-                "preferred_event_data": {"group": "Burst"},
+                "preferred_event_data": {"group": "Burst", "search": "AllSky"},
                 "far": 1e-5}
     elif graceid == "S2345":
         return {"superevent_id": "S2345",
                 "preferred_event": "G000002",
-                "preferred_event_data": {"group": "Burst"},
+                "preferred_event_data": {"group": "Burst", "search": "AllSky"},
                 "far": 1e-10}
     elif graceid == "S2468":
         return {"superevent_id": "S2468",
                 "preferred_event": "G000002",
-                "preferred_event_data": {"group": "CBC"},
+                "preferred_event_data": {"group": "CBC", "search": "AllSky"},
                 "far": 1e-4}
     elif graceid == "S5678":
         return {"superevent_id": "S5678",
                 "preferred_event": "G000003",
-                "preferred_event_data": {"group": "CBC"},
+                "preferred_event_data": {"group": "CBC", "search": "AllSky"},
                 "far": 1e-5}
     elif graceid == "S8642":
         return {"superevent_id": "S8642",
                 "preferred_event": "G000003",
-                "preferred_event_data": {"group": "Burst"},
+                "preferred_event_data": {"group": "Burst", "search": "AllSky"},
                 "far": 1e-3}
     elif graceid == "S9876":
         return {"superevent_id": "S9876",
                 "preferred_event": "G000003",
-                "preferred_event_data": {"group": "Burst"},
+                "preferred_event_data": {"group": "Burst", "search": "AllSky"},
                 "far": 1e-6}
-    elif graceid == "S9999":
-        return {"superevent_id": "S9876",
+    elif graceid == "S9988":
+        return {"superevent_id": "S9988",
                 "preferred_event": "G000003",
-                "preferred_event_data": {"group": "Burst"},
+                "preferred_event_data": {"group": "Burst", "search": "BBH"},
+                "far": 1e-7}
+    elif graceid == "S9999":
+        return {"superevent_id": "S9999",
+                "preferred_event": "G000003",
+                "preferred_event_data": {"group": "Burst", "search": "AllSky"},
                 "far": -1e-6}
     elif graceid == "TS1111":
         return {"superevent_id": "TS1111",
                 "preferred_event": "G000003",
-                "preferred_event_data": {"group": "Test"},
+                "preferred_event_data": {"group": "Test", "search": "AllSky"},
                 "far": 1e-6}
     elif graceid == "MS1111":
         return {"superevent_id": "MS1111",
                 "preferred_event": "M000004",
-                "preferred_event_data": {"group": "CBC"},
+                "preferred_event_data": {"group": "CBC", "search": "MDC"},
                 "far": 1e-9}
     elif graceid == 'E1':
         return {"graceid": "E1",
@@ -604,6 +683,9 @@ def _mock_get_coinc_far(graceid):
     elif graceid == "S9876":
         return {"temporal_coinc_far": 1e-07,
                 "spatiotemporal_coinc_far": 1e-13}
+    elif graceid == "S9988":
+        return {"temporal_coinc_far": 1e-09,
+                "spatiotemporal_coinc_far": 1e-14}
     elif graceid == "MS1111":
         return {"temporal_coinc_far": 1e-09,
                 "spatiotemporal_coinc_far": 1e-10}
@@ -629,6 +711,7 @@ def _return_coinc_far_dict(coinc_far_dict, *args):
      ['S5678', 'E1', 'CBC', False],
      ['S5678', 'E7', 'CBC', False],
      ['E1', 'S9876', 'CBC', True],
+     ['E1', 'S9988', 'Burst', True],
      ['E1', 'S2468', 'CBC', False],
      ['E2', 'S5678', 'CBC', False],
      ['E3', 'S8642', 'Burst', False],
