@@ -100,7 +100,7 @@ def typeahead_superevent_id():
     else:
         prefix = 'S'
         category = 'production'
-        date_end = datetime.datetime.utcnow()
+        date_end = datetime.datetime.now(datetime.timezone.utc)
         date_start = date_end - datetime.timedelta(days=7)
         date_partial = ''
         date_partial_length = 0
@@ -375,9 +375,13 @@ def send_update_gcn():
     """Handle submission of update alert form."""
     keys = ('superevent_id', 'skymap_filename',
             'em_bright_filename', 'p_astro_filename')
-    superevent_id, *filenames = args = tuple(
-        request.form.get(key) for key in keys)
-    if all(args):
+    # Get filenames and map empty strings to Nones so that the orchestrator
+    # will ignore them and not include these values in the alert
+    superevent_id, *filenames = tuple(
+        request.form.get(key) if request.form.get(key) != '' else None
+        for key in keys)
+    # Check at least superevent ID and sky map are present
+    if superevent_id is not None and filenames[0] is not None:
         (
             gracedb.upload.s(
                 None, None, superevent_id,
@@ -389,7 +393,7 @@ def send_update_gcn():
         ).delay()
         flash('Queued update alert for {}.'.format(superevent_id), 'success')
     else:
-        flash('No alert sent. Please fill in all fields.', 'danger')
+        flash('No alert sent. Please fill in all required fields.', 'danger')
     return redirect(url_for('index'))
 
 
@@ -503,7 +507,7 @@ def send_mock_event():
 @gracedb.task(shared=False)
 def _create_upload_external_event(gpstime):
     new_time = first2years_external._offset_time(
-        gpstime, 'CBC', 'Fermi', 'GRB')
+        gpstime, 'CBC', 'Fermi', 'GRB', 'MDC')
 
     ext_event = first2years_external.create_upload_external_event(
                     new_time, 'Fermi', 'MDC')
