@@ -141,8 +141,8 @@ class Receiver(TwistedService):
 
     A few :doc:`configuration options <configuration>` are available:
 
-    * ``voevent_receiver_address``: The address to connect to, in
-      :samp:`{host}:{port}` format.
+    * ``voevent_receiver_addresses``: A list of addresses to connect to, each
+      in :samp:`{host}:{port}` format.
 
     The list of active connections is made available :ref:`inspection
     <celery:worker-inspect>` with the ``gwcelery inspect stats`` command under
@@ -157,6 +157,7 @@ class Receiver(TwistedService):
     def create_service(self, consumer):
         from comet.icomet import IHandler
         from twisted.application.internet import TCPClient
+        from twisted.application.service import MultiService
         from zope.interface import implementer
 
         from .subscriber import VOEventSubscriberFactory
@@ -171,10 +172,20 @@ class Receiver(TwistedService):
 
         conf = consumer.app.conf
         local_ivo = get_local_ivo(consumer.app)
-        host, port = get_host_port(conf['voevent_receiver_address'])
         self._factory = factory = VOEventSubscriberFactory(
             local_ivo=local_ivo, handlers=[Handler()])
-        return TCPClient(host, port, factory)
+
+        # Create a MultiService to manage multiple TCP connections
+        multi_service = MultiService()
+
+        # Create a TCP client for each address
+        for address in conf['voevent_receiver_addresses']:
+            host, port = get_host_port(address)
+            tcp_client = TCPClient(host, port, factory)
+            tcp_client.setServiceParent(multi_service)
+            log.info('Configured VOEvent receiver for %s:%s', host, port)
+
+        return multi_service
 
     def info(self, consumer):
         try:
